import React, { useState, Suspense } from 'react';
import PropTypes from 'prop-types';
import styles from './ChatShell.module.scss';
import ChatListShell from '../ChatListShell/ChatListShell';
import ConversationShell from '../ConversationShell/ConversationShell';
import { chatListOptions } from '../../sampleData/chatListOptions';
import {
  ApolloClient,
  InMemoryCache,
  ApolloProvider,
  HttpLink,
  from,
  split,
} from '@apollo/client';
import { onError } from '@apollo/client/link/error';
import { WebSocketLink } from '@apollo/client/link/ws';
import { getMainDefinition } from '@apollo/client/utilities';
import { useParams } from 'react-router-dom';
import usePreviousState from '../../_helpers/usePreviousState';

const ChatShell = ({ connectionUri, subscriptionUri, chatApiUrl, query, ...rest }) => {
  const [queryMessagesBody, setQueryMessagesBody] = React.useState(null);
  const prevQueryRef = usePreviousState(query);
  
  React.useEffect(() => {
    if (
      query?.parameter.length === 0 ||
      prevQueryRef?.parameter !== query.parameter
    ) {
      setQueryMessagesBody(null);
    }
  }, [query, prevQueryRef]);

  const errorLink = onError(({ graphqlErrors, networkError }) => {
    if (graphqlErrors) {
      graphqlErrors.map(({ message, location, path }) => {
        return alert(`Graphql error ${message}`);
      });
    }
  });

  const httpLink = from([errorLink, new HttpLink({ uri: connectionUri })]);

  const wsLink = process.browser
    ? new WebSocketLink({
        uri: subscriptionUri,
        options: {
          reconnect: true,
          lazy: true,
        },
      })
    : null;

  const splitLink = process.browser
    ? split(
        ({ query }) => {
          const definition = getMainDefinition(query);
          return (
            definition.kind === 'OperationDefinition' &&
            definition.operation === 'subscription'
          );
        },
        wsLink,
        httpLink
      )
    : httpLink;

  const client = new ApolloClient({
    cache: new InMemoryCache(),
    link: splitLink,
  });

  const [isUsername, setIsUsername] = useState(false)

  const handleSetIsUsername = (val) => {
    if (val) {
      setIsUsername(true)
    } else {
      setIsUsername(false)
    }
  }

  const handleQueryMessage = (messageBody) => {
    setQueryMessagesBody(messageBody);
  };

  return (
    <ApolloProvider client={client}>
      <div className={styles.ChatShell}>
        <ChatListShell
          options={chatListOptions}
          isUsername={isUsername}
          query={query}
          setQueryMessagesBody={handleQueryMessage}
          {...rest}
        />
        <ConversationShell chatApiUrl={chatApiUrl} queryMessagesBody={queryMessagesBody} handleSetIsUsername={handleSetIsUsername} />
      </div>
    </ApolloProvider>
  );
};

ChatShell.propTypes = {
  query: PropTypes.shape({
    parameter: PropTypes.string,
  }),
  resetChatList: PropTypes.func,
};

ChatShell.defaultProps = {
  query: { parameter: '' },
  resetChatList: () =>
    console.log(
      'This function will reset the ChatListSheel to its default state'
    ),
};

export default React.memo(ChatShell);
